import * as jsforce from 'jsforce';
require('dotenv').config();
import { GraphQLServer } from 'graphql-yoga';
import { IDirectiveResolvers } from 'graphql-tools';
import * as express from 'express';
import * as bodyParser from 'body-parser';
import  { graphqlExpress, graphiqlExpress } from 'apollo-server-express';
import  { makeExecutableSchema } from 'graphql-tools';
import * as path from 'path';
import * as fs from 'fs';
const schemaFile = path.join(__dirname, 'schema.graphql');
const typeDefs = fs.readFileSync(schemaFile, 'utf8');

import { Salesforce } from '../src';

const resolvers = {
  Query: {
    Accounts: (parent, args, context, info) => context.db.query({}, info).then(res => res.records),
    Account: (parent, args, context, info) =>
      context.sf.query({}, info).then(res => res.records[0]),
    Contacts: (parent, args, context, info) => context.db.query({}, info).then(res => res.records),
    Contact: (parentobj, args, context, info) =>
      context.sf.query({}, info).then(res => res.records[0]),
  },
  Account: {
    Contacts: (parent, args, context, info) =>
      context.sf.query({ AccountId: parent.Id }, info).then(res => res.records),
  },
  Contact: {
    Account: (parent, args, context, info) =>
      context.sf.query({ Id: parent.AccountId }, info).then(res => res.records[0]),
  },
};

const sf = new jsforce.Connection({});
sf.login(process.env.SALESFORCE_USERNAME, process.env.SALESFORCE_PASSWORD, (err, userinfo) => {
  init();
});

function init() {
  const client = new Salesforce({ sf });
  // Put together a schema
  const server = new GraphQLServer({
    typeDefs,
    resolvers,
    context: req => ({ ...req, sf }),
  });

  server.start({ playground: '/playground' }, ({ port }) =>
    console.log('Server is running on localhost:' + port)
  );
}
